@@ -1,8 +1,9 @@
-package XML::Sig;
-
 use strict;
 use warnings;
 
+package XML::Sig;
+our $VERSION = '0.60';
+
 use Encode;
 # ABSTRACT: XML::Sig - A toolkit to help sign and verify XML Digital Signatures
 
@@ -10,7 +11,6 @@ use Encode;
 use vars qw($VERSION @EXPORT_OK %EXPORT_TAGS $DEBUG);
 
 $DEBUG = 0;
-our $VERSION = '0.54';
 
 use base qw(Class::Accessor);
 XML::Sig->mk_accessors(qw(key));
@@ -33,8 +33,6 @@ use constant TRANSFORM_EXC_C14N_COMMENTS => 'http://www.w3.org/2001/10/xml-exc-c
 
 sub DESTROY { }
 
-$SIG{INT} = sub { die "Interrupted\n"; };
-
 $| = 1;  # autoflush
 
 
@@ -44,7 +42,7 @@ sub new {
     my $class = shift;
     my $params = shift;
     my $self = {};
-    foreach my $prop ( qw/ key cert cert_text / ) {
+    foreach my $prop ( qw/ key cert cert_text ns id_attr/ ) {
         if ( exists $params->{ $prop } ) {
             $self->{ $prop } = $params->{ $prop };
         }
@@ -54,6 +52,9 @@ sub new {
     }
     bless $self, $class;
     $self->{ 'x509' } = exists $params->{ x509 } ? 1 : 0;
+    if ( exists $params->{ key_name } ) {
+        $self->{ key_name } = $params->{ key_name };
+    }
     if ( exists $params->{ 'key' } ) {
         $self->_load_key( $params->{ 'key' } );
     }
@@ -63,6 +64,9 @@ sub new {
     if ( exists $params->{ 'cert_text' } ) {
         $self->_load_cert_text( $params->{ 'cert_text' } );
     }
+    if ( exists $params->{ 'hmac_key' } ) {
+        $self->_load_hmac_key_info;
+    }
 
     if ( exists $params->{ sig_hash } && grep { $_ eq $params->{ sig_hash } } ('sha224', 'sha256', 'sha384', 'sha512', 'ripemd160'))
     {
@@ -81,10 +85,12 @@ sub new {
     }
 
     if (defined $self->{ key_type } && $self->{ key_type } eq 'dsa') {
-        if ( defined $params->{ sig_hash } && grep { $_ eq $params->{ sig_hash } } ('sha1', 'sha256')) {
-            $self->{ sig_hash } = $params->{ sig_hash };
-        }
-        else {
+        my $sig_size = $self->{ key_obj }->get_sig_size();
+
+        # The key size dictates the sig size
+        if ( $sig_size eq 48 ) {    # 1024-bit key
+            $self->{ sig_hash } = 'sha1';
+        } else {                    # 2048-bit or 3072-bit key
             $self->{ sig_hash } = 'sha256';
         }
     }
@@ -118,13 +124,20 @@ sub sign {
     $self->{ parser }->registerNs('dsig', 'http://www.w3.org/2000/09/xmldsig#');
     $self->{ parser }->registerNs('ec', 'http://www.w3.org/2001/10/xml-exc-c14n#');
     $self->{ parser }->registerNs('saml', 'urn:oasis:names:tc:SAML:2.0:assertion');
+    if ($self->{ns}) {
+        foreach (keys %{$self->{ns}}) {
+            $self->{ parser }->registerNs($_, $self->{ns}{$_});
+        }
+    }
 
     print ("Signing XML\n") if $DEBUG;
 
     my @ids_to_sign = $self->_get_ids_to_sign();
 
-    foreach (@ids_to_sign) {
-        my $signid = $_;
+    foreach my $signid (@ids_to_sign) {
+
+        print ("Signing ID $signid\n") if $DEBUG;
+
         # Temporarily create the Signature XML from the part
         # TODO: ths section needs a rewrite to create the xml in
         # a better way.
@@ -428,6 +441,17 @@ sub signer_cert {
 ##
 sub _get_ids_to_sign {
     my $self = shift;
+
+    if ($self->{id_attr}) {
+        my $nodes = $self->{parser}->findnodes($self->{id_attr});
+        if ($nodes->size == 0) {
+            die "Unable to find an attribute node with $self->{id_attr}";
+        }
+        my $node = $nodes->get_node(1);
+        return $node->getAttribute('ID');
+
+    }
+
     my @id = $self->{parser}->findnodes('//@ID');
     my @ids;
     foreach (@id) {
@@ -1054,7 +1078,7 @@ sub _load_ecdsa_key {
     or confess "Crypt::PK::ECC 0.036+ needs to be installed so
              that we can handle ECDSA signatures";
 
-    my $ecdsa_key = Crypt::PK::ECC->new('t/ecdsa.private.pem');
+    my $ecdsa_key = Crypt::PK::ECC->new(\$key_text);
 
     if ( $ecdsa_key ) {
         $self->{ key_obj } = $ecdsa_key;
@@ -1186,6 +1210,27 @@ sub _load_rsa_key {
     }
 }
 
+##
+## _load_hmac_key_info()
+##
+## Arguments:
+##    none
+##
+## Returns: nothing
+##
+## Populate:
+##   self->{KeyInfo}
+##
+sub _load_hmac_key_info {
+    my $self = shift;
+
+    if (! defined $self->{ key_name }) {
+        return;
+    }
+
+    $self->{KeyInfo} = qq{<dsig:KeyInfo><dsig:KeyName>$self->{key_name}</dsig:KeyName></dsig:KeyInfo>};
+}
+
 ##
 ## _load_x509_key($key_text)
 ##
@@ -1365,10 +1410,6 @@ sub _signature_xml {
     my $self = shift;
     my ($signed_info,$signature_value) = @_;
 
-    if (! defined $self->{KeyInfo} && defined $self->{ hmac_key }) {
-        $self->{KeyInfo} = '';
-    }
-
     return qq{<dsig:Signature xmlns:dsig="http://www.w3.org/2000/09/xmldsig#">
             $signed_info
             <dsig:SignatureValue>$signature_value</dsig:SignatureValue>
@@ -1653,7 +1694,7 @@ XML::Sig - XML::Sig - A toolkit to help sign and verify XML Digital Signatures
 
 =head1 VERSION
 
-version 0.54
+version 0.60
 
 =head1 SYNOPSIS
 
@@ -1785,7 +1826,9 @@ hashing algorithm used when signing the SignedInfo.  RSA and ECDSA
 supports the hashes specified sha1, sha224, sha256, sha384 and sha512
 
 DSA supports only sha1 and sha256 (but you really should not sign
-anything with DSA anyway).
+anything with DSA anyway).  This is over-ridden by the key's signature
+size which is related to the key size.  1024-bit keys require sha1,
+2048-bit and 3072-bit keys require sha256.
 
 =item B<digest_hash>
 
@@ -1798,6 +1841,12 @@ sha384, sha512, ripemd160
 
 Base64 encoded hmac_key
 
+=item B<key_name>
+
+The name of the key that should be referenced.  In the case of
+xmlsec the --keys-file (ex. t/xmlsec-keys.xml) holds keys with a
+KeyName that is referenced by this name.
+
 =item B<no_xml_declaration>
 
 Some applications such as Net::SAML2 expect to sign a fragment of the
@@ -1807,6 +1856,21 @@ undefined returns an XML document starting with the XML Declaration.
 
 =back
 
+The following options act similar to C<< xmlsec --id-attr:ID
+<node-namespace-uri>:<name> >>
+
+=over
+
+=item B<ns>
+
+A HashRef to namespaces you want to define to select the correct attribute ID on
+
+=item B<id_attr>
+
+The xpath string you want to sign your XML message on.
+
+=back
+
 =head2 METHODS
 
 =head3 B<new(...)>
@@ -1988,7 +2052,7 @@ Timothy Legge <timlegge@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Byrne Reese, Chris Andrews and Others; in detail:
+This software is copyright (c) 2023 by Byrne Reese, Chris Andrews and Others; in detail:
 
   Copyright 2009       Byrne, Michael Hendricks
             2010       Chris Andrews
@@ -1998,6 +2062,8 @@ This software is copyright (c) 2021 by Byrne Reese, Chris Andrews and Others; in
             2016       Jeff Fearn
             2017       Mike Wisener, xmikew
             2019-2021  Timothy Legge
+            2022       Timothy Legge, Wesley Schwengle
+            2023       Timothy Legge
 
 
 This is free software; you can redistribute it and/or modify it under
